@@ -48,6 +48,13 @@ struct TickitTerm {
   TermKey              *termkey;
   struct timeval        input_timeout_at; /* absolute time */
 
+  char *outbuffer;
+  size_t outbuffer_len; /* size of outbuffer */
+  size_t outbuffer_cur; /* current fill level */
+
+  char *tmpbuffer;
+  size_t tmpbuffer_len;
+
   struct {
     unsigned int bce:1;
   } cap;
@@ -90,6 +97,8 @@ TickitTerm *tickit_term_new(void)
   return tickit_term_new_for_termtype(termtype);
 }
 
+void tickit_term_free(TickitTerm *tt);
+
 TickitTerm *tickit_term_new_for_termtype(const char *termtype)
 {
   TickitTerm *tt = malloc(sizeof(TickitTerm));
@@ -104,6 +113,13 @@ TickitTerm *tickit_term_new_for_termtype(const char *termtype)
   tt->termkey_flags = 0;
   tt->input_timeout_at.tv_sec = -1;
 
+  tt->outbuffer = NULL;
+  tt->outbuffer_len = 0;
+  tt->outbuffer_cur = 0;
+
+  tt->tmpbuffer = NULL;
+  tt->tmpbuffer_len = 0;
+
   tt->mode.altscreen = 0;
   tt->mode.cursorvis = 1;
   tt->mode.mouse     = 0;
@@ -157,6 +173,12 @@ void tickit_term_free(TickitTerm *tt)
   if(tt->termkey)
     termkey_destroy(tt->termkey);
 
+  if(tt->outbuffer)
+    free(tt->outbuffer);
+
+  if(tt->tmpbuffer)
+    free(tt->tmpbuffer);
+
   free(tt);
 }
 
@@ -172,6 +194,18 @@ void tickit_term_destroy(TickitTerm *tt)
   tickit_term_free(tt);
 }
 
+static void *get_tmpbuffer(TickitTerm *tt, size_t len)
+{
+  if(tt->tmpbuffer_len < len) {
+    if(tt->tmpbuffer)
+      free(tt->tmpbuffer);
+    tt->tmpbuffer = malloc(len);
+    tt->tmpbuffer_len = len;
+  }
+
+  return tt->tmpbuffer;
+}
+
 void tickit_term_get_size(TickitTerm *tt, int *lines, int *cols)
 {
   if(lines)
@@ -221,6 +255,18 @@ void tickit_term_set_output_func(TickitTerm *tt, TickitTermOutputFunc *fn, void
   tt->outfunc_user = user;
 }
 
+void tickit_term_set_output_buffer(TickitTerm *tt, size_t len)
+{
+  void *buffer = len ? malloc(len) : NULL;
+
+  if(tt->outbuffer)
+    free(tt->outbuffer);
+
+  tt->outbuffer = buffer;
+  tt->outbuffer_len = len;
+  tt->outbuffer_cur = 0;
+}
+
 void tickit_term_set_input_fd(TickitTerm *tt, int fd)
 {
   if(tt->termkey)
@@ -235,6 +281,11 @@ int tickit_term_get_input_fd(TickitTerm *tt)
   return tt->infd;
 }
 
+int tickit_term_get_utf8(TickitTerm *tt)
+{
+  return tt->termkey_flags & TERMKEY_FLAG_UTF8;
+}
+
 void tickit_term_set_utf8(TickitTerm *tt, int utf8)
 {
   /* TODO: See what we can think of for the output side */
@@ -383,12 +434,38 @@ void tickit_term_input_wait(TickitTerm *tt)
   }
 }
 
+void tickit_term_flush(TickitTerm *tt)
+{
+  if(tt->outbuffer_cur == 0)
+    return;
+
+  if(tt->outfunc)
+    (*tt->outfunc)(tt, tt->outbuffer, tt->outbuffer_cur, tt->outfunc_user);
+  else if(tt->outfd != -1) {
+    write(tt->outfd, tt->outbuffer, tt->outbuffer_cur);
+  }
+
+  tt->outbuffer_cur = 0;
+}
+
 static void write_str(TickitTerm *tt, const char *str, size_t len)
 {
   if(len == 0)
     len = strlen(str);
 
-  if(tt->outfunc) {
+  if(tt->outbuffer) {
+    while(len > 0) {
+      size_t space = tt->outbuffer_len - tt->outbuffer_cur;
+      if(len < space)
+        space = len;
+      memcpy(tt->outbuffer + tt->outbuffer_cur, str, space);
+      tt->outbuffer_cur += space;
+      len -= space;
+      if(tt->outbuffer_cur >= tt->outbuffer_len)
+        tickit_term_flush(tt);
+    }
+  }
+  else if(tt->outfunc) {
     (*tt->outfunc)(tt, str, len, tt->outfunc_user);
   }
   else if(tt->outfd != -1) {
@@ -404,20 +481,18 @@ static void write_str_rep(TickitTerm *tt, const char *str, size_t len, int repea
   if(repeat < 1)
     return;
 
-  if(tt->outfunc) {
-    char *buffer = malloc(len * repeat + 1);
+  if(tt->outfunc && !tt->outbuffer) {
+    char *buffer = get_tmpbuffer(tt, len * repeat + 1);
     char *s = buffer;
     for(int i = 0; i < repeat; i++) {
       strncpy(s, str, len);
       s += len;
     }
     (*tt->outfunc)(tt, buffer, len * repeat, tt->outfunc_user);
-    free(buffer);
   }
-  else if(tt->outfd != -1) {
-    for(int i = 0; i < repeat; i++) {
-      write(tt->outfd, str, len);
-    }
+  else {
+    for(int i = 0; i < repeat; i++)
+      write_str(tt, str, len);
   }
 }
 
@@ -432,12 +507,10 @@ static void write_vstrf(TickitTerm *tt, const char *fmt, va_list args)
     return;
   }
 
-  char *morebuffer = malloc(len + 1);
+  char *morebuffer = get_tmpbuffer(tt, len + 1);
   vsnprintf(morebuffer, len + 1, fmt, args);
 
   write_str(tt, morebuffer, len);
-
-  free(morebuffer);
 }
 
 static void write_strf(TickitTerm *tt, const char *fmt, ...)
@@ -627,7 +700,7 @@ static void do_pen(TickitTerm *tt, TickitPen *pen, int ignoremissing)
   if(pindex > 0)
     len--; /* Last one has no final separator */
 
-  char *buffer = malloc(len + 1);
+  char *buffer = get_tmpbuffer(tt, len + 1);
   char *s = buffer;
 
   s += sprintf(s, "\e[");
@@ -639,8 +712,6 @@ static void do_pen(TickitTerm *tt, TickitPen *pen, int ignoremissing)
   sprintf(s, "m");
 
   write_str(tt, buffer, len);
-
-  free(buffer);
 }
 
 void tickit_term_chpen(TickitTerm *tt, TickitPen *pen)
